# coding: utf-8
from datetime import datetime
from random import randint
from names import KINDERGARTEN_NAMES


def get_id():
    return datetime.now().strftime('%y%m%d%H%M%S%f')


class Organization(object):
    u"""
    Класс для формирования данных по организации.
    """
    def __init__(self, region, city, oktmo, system, factory):
        self.id = get_id()
        self.name = self._get_name(system, factory)
        self.region = region
        self.city = city
        self.oktmo = oktmo
        self.inn = self._get_inn()
        self.kpp = randint(9000000, 9999999)
        self.ogrn = self._get_ogrn()
        self.other_id = get_id()
        self.prof_id = get_id()

    @staticmethod
    def _get_inn():
        factors = [2, 4, 10, 3, 5, 9, 4, 6, 8]
        numbers = [randint(1, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        numbers = sum([factors[i]*numbers[i] for i in xrange(len(numbers))])
        check_sum = str(numbers % 11) if numbers % 11 != 10 else '0'
        # if check_sum == '10':
        #     check_sum = '0'

        return ''.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_ogrn():
        num = randint(9100000000, 9999999999)
        return ''.join([str((num*11)+3), '3'])
    
    @staticmethod
    def _get_name(system, factory):
        if system.lower() == 'dou':
            return KINDERGARTEN_NAMES[randint(0, len(KINDERGARTEN_NAMES)-1)]
        else:
            return factory.company().split(' ')[0].replace(',', '')


def mult(inn, factors):
    return [int(inn[i])*factors[i] for i in xrange(len(factors))]


class Person(object):
    u"""
    Класс для формирования данных по персоне.
    """
    def __init__(
            self, firstname, middlename, lastname,
            delegate_firstname, delegate_middlename,
            delegate_lastname, organization, system
    ):
        self.id = get_id()
        self.firstname = firstname
        self.middlename = middlename
        self.lastname = lastname
        self.snils = self._get_snils()
        self.document_id, self.document_series, \
            self.document_number = self._get_document(system, is_child=True)
        self.decl_id = get_id()
        self.decl_org_id = get_id()
        self.other_id = get_id()
        self.delegate_id = get_id()
        self.delegate_firstname = delegate_firstname
        self.delegate_middlename = delegate_middlename
        self.delegate_lastname = delegate_lastname
        self.delegate_snils = self._get_snils()
        self.delegate_doc_id, self.delegate_doc_series, \
            self.delegate_doc_number = self._get_document(system, is_child=False)
        self.delegate_person_id = get_id()
        self.organization = organization
        self.edu_id = get_id()
        self.project_id = get_id()
        self.org_activities_id = get_id()
        self.activities_id = get_id()
        self.achieve_id = get_id()
        self.ex_activities_id = get_id()
        self.self_stud_id = get_id()
        self.main_edu_class = get_id()
        self.basic_edu_mark_id = get_id()
        self.common_edu_mark_id = get_id()
        self.mid_edu_id = get_id()
        self.ann_mark_id = get_id()
        self.final_mark_id = get_id()
        self.employment_id = get_id()
        self.prof_edu_id = get_id()
        self.addit_edu_id = get_id()
        self.addit_edu_score_id = get_id()

    @staticmethod
    def _get_snils():
        numbers = [randint(0, 9) for _ in xrange(9)]
        str_num = [str(n) for n in numbers]
        multipls = list(xrange(1, 10))
        multipls.reverse()
        numbers = [numbers[i]*multipls[i] for i in xrange(len(numbers))]
        num_sum = sum(numbers)
        if num_sum == 100:
            check_sum = '00'
        else:
            check_sum = '%02d' % (num_sum % 101)

        for i in (3, 7):
            str_num.insert(i, '-')

        return ' '.join([''.join(str_num), check_sum])

    @staticmethod
    def _get_document(system, is_child=True):
        doc_id = get_id()
        SERIES_DICT = {
            1: u'V-ТФ',
            2: u'V-АГ'
        }
        if is_child and (system == 'dou' or system == 'school'):
            series = SERIES_DICT[randint(1, 2)]
        else:
            series = int('%04d' % randint(0, 9999))
        number = '%06d'.encode('utf-8') % randint(0, 999999)
        return doc_id, series, number
